@@ -26,7 +26,6 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-#if NET_2_0
 using System.Collections;
 using System.Collections.Generic;
 using System.Runtime.InteropServices;
@@ -93,6 +92,7 @@ namespace System.Net.NetworkInformation {
 		}
 	}
 
+#if !MOBILE
 	class Win32GatewayIPAddressInformationCollection : GatewayIPAddressInformationCollection
 	{
 		public static readonly Win32GatewayIPAddressInformationCollection Empty = new Win32GatewayIPAddressInformationCollection (true);
@@ -128,19 +128,20 @@ namespace System.Net.NetworkInformation {
 			get { return is_readonly; }
 		}
 	}
+#endif
 
-	class LinuxGatewayIPAddressInformationCollection : GatewayIPAddressInformationCollection
+	class UnixGatewayIPAddressInformationCollection : GatewayIPAddressInformationCollection
 	{
-		public static readonly LinuxGatewayIPAddressInformationCollection Empty = new LinuxGatewayIPAddressInformationCollection (true);
+		public static readonly UnixGatewayIPAddressInformationCollection Empty = new UnixGatewayIPAddressInformationCollection (true);
 
 		bool is_readonly;
 
-		private LinuxGatewayIPAddressInformationCollection (bool isReadOnly)
+		private UnixGatewayIPAddressInformationCollection (bool isReadOnly)
 		{
 			this.is_readonly = isReadOnly;
 		}
 
-		public LinuxGatewayIPAddressInformationCollection (IPAddressCollection col)
+		public UnixGatewayIPAddressInformationCollection (IPAddressCollection col)
 		{
 			foreach (IPAddress a in col)
 				Add (new GatewayIPAddressInformationImpl (a));
@@ -152,5 +153,4 @@ namespace System.Net.NetworkInformation {
 		}
 	}
 }
-#endif
 
