@@ -12,15 +12,12 @@
 #include <string.h>
 #include <errno.h>
 
-#include <mono/os/gc_wrapper.h>
-
 #include <mono/io-layer/wapi.h>
 #include <mono/io-layer/handles-private.h>
 #include <mono/io-layer/wapi-private.h>
-#include <mono/io-layer/mono-mutex.h>
-#include <mono/io-layer/misc-private.h>
-
-#undef DEBUG
+#include <mono/io-layer/io-trace.h>
+#include <mono/utils/mono-logger-internals.h>
+#include <mono/utils/mono-time.h>
 
 static gboolean own_if_signalled(gpointer handle)
 {
@@ -89,10 +86,10 @@ guint32 WaitForSingleObjectEx(gpointer handle, guint32 timeout,
 			      gboolean alertable)
 {
 	guint32 ret, waited;
-	struct timespec abstime;
 	int thr_ret;
 	gboolean apc_pending = FALSE;
-	gpointer current_thread = _wapi_thread_handle_from_id (pthread_self ());
+	gpointer current_thread = wapi_get_current_thread_handle ();
+	gint64 now, end;
 	
 	if (current_thread == NULL) {
 		SetLastError (ERROR_INVALID_HANDLE);
@@ -100,7 +97,7 @@ guint32 WaitForSingleObjectEx(gpointer handle, guint32 timeout,
 	}
 
 	if (handle == _WAPI_THREAD_CURRENT) {
-		handle = _wapi_thread_handle_from_id (pthread_self ());
+		handle = wapi_get_current_thread_handle ();
 		if (handle == NULL) {
 			SetLastError (ERROR_INVALID_HANDLE);
 			return(WAIT_FAILED);
@@ -114,10 +111,8 @@ guint32 WaitForSingleObjectEx(gpointer handle, guint32 timeout,
 	
 	if (_wapi_handle_test_capabilities (handle,
 					    WAPI_HANDLE_CAP_WAIT) == FALSE) {
-#ifdef DEBUG
-		g_message ("%s: handle %p can't be waited for", __func__,
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: handle %p can't be waited for", __func__,
 			   handle);
-#endif
 
 		return(WAIT_FAILED);
 	}
@@ -125,53 +120,35 @@ guint32 WaitForSingleObjectEx(gpointer handle, guint32 timeout,
 	_wapi_handle_ops_prewait (handle);
 	
 	if (_wapi_handle_test_capabilities (handle, WAPI_HANDLE_CAP_SPECIAL_WAIT) == TRUE) {
-#ifdef DEBUG
-		g_message ("%s: handle %p has special wait", __func__, handle);
-#endif
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: handle %p has special wait", __func__, handle);
 
-		ret = _wapi_handle_ops_special_wait (handle, timeout);
+		ret = _wapi_handle_ops_special_wait (handle, timeout, alertable);
 	
-		if (alertable && _wapi_thread_apc_pending (current_thread)) {
-			apc_pending = TRUE;
+		if (alertable && _wapi_thread_cur_apc_pending ())
 			ret = WAIT_IO_COMPLETION;
-		}
 
-		goto check_pending;
+		return ret;
 	}
 	
 	
-#ifdef DEBUG
-	g_message ("%s: locking handle %p", __func__, handle);
-#endif
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: locking handle %p", __func__, handle);
 
-	pthread_cleanup_push ((void(*)(void *))_wapi_handle_unlock_handle,
-			      handle);
 	thr_ret = _wapi_handle_lock_handle (handle);
 	g_assert (thr_ret == 0);
 
 	if (_wapi_handle_test_capabilities (handle,
 					    WAPI_HANDLE_CAP_OWN) == TRUE) {
 		if (own_if_owned (handle) == TRUE) {
-#ifdef DEBUG
-			g_message ("%s: handle %p already owned", __func__,
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: handle %p already owned", __func__,
 				   handle);
-#endif
 			ret = WAIT_OBJECT_0;
 			goto done;
 		}
 	}
-	
-	if (alertable && _wapi_thread_apc_pending (current_thread)) {
-		apc_pending = TRUE;
-		ret = WAIT_IO_COMPLETION;
-		goto done;
-	}
-	
+
 	if (own_if_signalled (handle) == TRUE) {
-#ifdef DEBUG
-		g_message ("%s: handle %p already signalled", __func__,
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: handle %p already signalled", __func__,
 			   handle);
-#endif
 
 		ret=WAIT_OBJECT_0;
 		goto done;
@@ -181,34 +158,34 @@ guint32 WaitForSingleObjectEx(gpointer handle, guint32 timeout,
 		ret = WAIT_TIMEOUT;
 		goto done;
 	}
-	/* Have to wait for it */
-	if (timeout != INFINITE) {
-		_wapi_calc_timeout (&abstime, timeout);
-	}
 	
+	if (timeout != INFINITE)
+		end = mono_100ns_ticks () + timeout * 1000 * 10;
+
 	do {
 		/* Check before waiting on the condition, just in case
 		 */
 		_wapi_handle_ops_prewait (handle);
 
 		if (own_if_signalled (handle)) {
-#ifdef DEBUG
-			g_message ("%s: handle %p signalled", __func__,
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: handle %p signalled", __func__,
 				   handle);
-#endif
 
 			ret = WAIT_OBJECT_0;
 			goto done;
 		}
-			
+
 		if (timeout == INFINITE) {
-			waited = _wapi_handle_wait_signal_handle (handle, alertable);
+			waited = _wapi_handle_timedwait_signal_handle (handle, INFINITE, alertable, FALSE, &apc_pending);
 		} else {
-			waited = _wapi_handle_timedwait_signal_handle (handle, &abstime, alertable, FALSE);
+			now = mono_100ns_ticks ();
+			if (end < now) {
+				ret = WAIT_TIMEOUT;
+				goto done;
+			}
+
+			waited = _wapi_handle_timedwait_signal_handle (handle, (end - now) / 10 / 1000, alertable, FALSE, &apc_pending);
 		}
-	
-		if (alertable)
-			apc_pending = _wapi_thread_apc_pending (current_thread);
 
 		if(waited==0 && !apc_pending) {
 			/* Condition was signalled, so hopefully
@@ -216,10 +193,8 @@ guint32 WaitForSingleObjectEx(gpointer handle, guint32 timeout,
 			 * if someone else got in before us.)
 			 */
 			if (own_if_signalled (handle)) {
-#ifdef DEBUG
-				g_message ("%s: handle %p signalled", __func__,
+				MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: handle %p signalled", __func__,
 					   handle);
-#endif
 
 				ret=WAIT_OBJECT_0;
 				goto done;
@@ -230,29 +205,18 @@ guint32 WaitForSingleObjectEx(gpointer handle, guint32 timeout,
 	} while(waited == 0 && !apc_pending);
 
 	/* Timeout or other error */
-#ifdef DEBUG
-	g_message ("%s: wait on handle %p error: %s", __func__, handle,
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: wait on handle %p error: %s", __func__, handle,
 		   strerror (waited));
-#endif
 
-	ret = WAIT_TIMEOUT;
-	
+	ret = apc_pending ? WAIT_IO_COMPLETION : WAIT_TIMEOUT;
+
 done:
 
-#ifdef DEBUG
-	g_message ("%s: unlocking handle %p", __func__, handle);
-#endif
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: unlocking handle %p", __func__, handle);
 	
 	thr_ret = _wapi_handle_unlock_handle (handle);
 	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
-	
-check_pending:
-	if (apc_pending) {
-		_wapi_thread_dispatch_apc_queue (current_thread);
-		ret = WAIT_IO_COMPLETION;
-	}
-		
+
 	return(ret);
 }
 
@@ -300,11 +264,11 @@ guint32 WaitForSingleObject(gpointer handle, guint32 timeout)
 guint32 SignalObjectAndWait(gpointer signal_handle, gpointer wait,
 			    guint32 timeout, gboolean alertable)
 {
-	guint32 ret, waited;
-	struct timespec abstime;
+	guint32 ret = 0, waited;
 	int thr_ret;
 	gboolean apc_pending = FALSE;
-	gpointer current_thread = _wapi_thread_handle_from_id (pthread_self ());
+	gpointer current_thread = wapi_get_current_thread_handle ();
+	gint64 now, end;
 	
 	if (current_thread == NULL) {
 		SetLastError (ERROR_INVALID_HANDLE);
@@ -312,7 +276,7 @@ guint32 SignalObjectAndWait(gpointer signal_handle, gpointer wait,
 	}
 
 	if (signal_handle == _WAPI_THREAD_CURRENT) {
-		signal_handle = _wapi_thread_handle_from_id (pthread_self ());
+		signal_handle = wapi_get_current_thread_handle ();
 		if (signal_handle == NULL) {
 			SetLastError (ERROR_INVALID_HANDLE);
 			return(WAIT_FAILED);
@@ -320,7 +284,7 @@ guint32 SignalObjectAndWait(gpointer signal_handle, gpointer wait,
 	}
 
 	if (wait == _WAPI_THREAD_CURRENT) {
-		wait = _wapi_thread_handle_from_id (pthread_self ());
+		wait = wapi_get_current_thread_handle ();
 		if (wait == NULL) {
 			SetLastError (ERROR_INVALID_HANDLE);
 			return(WAIT_FAILED);
@@ -356,12 +320,8 @@ guint32 SignalObjectAndWait(gpointer signal_handle, gpointer wait,
 		return (WAIT_FAILED);
 	}
 
-#ifdef DEBUG
-	g_message ("%s: locking handle %p", __func__, wait);
-#endif
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: locking handle %p", __func__, wait);
 
-	pthread_cleanup_push ((void(*)(void *))_wapi_handle_unlock_handle,
-			      wait);
 	thr_ret = _wapi_handle_lock_handle (wait);
 	g_assert (thr_ret == 0);
 
@@ -369,57 +329,45 @@ guint32 SignalObjectAndWait(gpointer signal_handle, gpointer wait,
 
 	if (_wapi_handle_test_capabilities (wait, WAPI_HANDLE_CAP_OWN)==TRUE) {
 		if (own_if_owned (wait)) {
-#ifdef DEBUG
-			g_message ("%s: handle %p already owned", __func__,
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: handle %p already owned", __func__,
 				   wait);
-#endif
 			ret = WAIT_OBJECT_0;
 			goto done;
 		}
 	}
-	
-	if (alertable && _wapi_thread_apc_pending (current_thread)) {
-		apc_pending = TRUE;
-		ret = WAIT_IO_COMPLETION;
-		goto done;
-	}
-	
+
 	if (own_if_signalled (wait)) {
-#ifdef DEBUG
-		g_message ("%s: handle %p already signalled", __func__, wait);
-#endif
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: handle %p already signalled", __func__, wait);
 
 		ret = WAIT_OBJECT_0;
 		goto done;
 	}
 
-	/* Have to wait for it */
-	if (timeout != INFINITE) {
-		_wapi_calc_timeout (&abstime, timeout);
-	}
-	
+	if (timeout != INFINITE)
+		end = mono_100ns_ticks () + timeout * 1000 * 10;
+
 	do {
 		/* Check before waiting on the condition, just in case
 		 */
 		_wapi_handle_ops_prewait (wait);
 	
 		if (own_if_signalled (wait)) {
-#ifdef DEBUG
-			g_message ("%s: handle %p signalled", __func__, wait);
-#endif
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: handle %p signalled", __func__, wait);
 
 			ret = WAIT_OBJECT_0;
 			goto done;
 		}
-		
-		if (timeout == INFINITE) {
-			waited = _wapi_handle_wait_signal_handle (wait, alertable);
-		} else {
-			waited = _wapi_handle_timedwait_signal_handle (wait, &abstime, alertable, FALSE);
-		}
 
-		if (alertable) {
-			apc_pending = _wapi_thread_apc_pending (current_thread);
+		if (timeout == INFINITE) {
+			waited = _wapi_handle_timedwait_signal_handle (wait, INFINITE, alertable, FALSE, &apc_pending);
+		} else {
+			now = mono_100ns_ticks ();
+			if (end < now) {
+				ret = WAIT_TIMEOUT;
+				goto done;
+			}
+
+			waited = _wapi_handle_timedwait_signal_handle (wait, (end - now) / 10 / 1000, alertable, FALSE, &apc_pending);
 		}
 
 		if (waited==0 && !apc_pending) {
@@ -428,10 +376,8 @@ guint32 SignalObjectAndWait(gpointer signal_handle, gpointer wait,
 			 * if someone else got in before us.)
 			 */
 			if (own_if_signalled (wait)) {
-#ifdef DEBUG
-				g_message ("%s: handle %p signalled", __func__,
+				MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: handle %p signalled", __func__,
 					   wait);
-#endif
 
 				ret = WAIT_OBJECT_0;
 				goto done;
@@ -442,59 +388,29 @@ guint32 SignalObjectAndWait(gpointer signal_handle, gpointer wait,
 	} while(waited == 0 && !apc_pending);
 
 	/* Timeout or other error */
-#ifdef DEBUG
-	g_message ("%s: wait on handle %p error: %s", __func__, wait,
-		   strerror (ret));
-#endif
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: wait on handle %p error: %s", __func__, wait, strerror (ret));
+
+	ret = apc_pending ? WAIT_IO_COMPLETION : WAIT_TIMEOUT;
 
-	ret = WAIT_TIMEOUT;
-	
 done:
 
-#ifdef DEBUG
-	g_message ("%s: unlocking handle %p", __func__, wait);
-#endif
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: unlocking handle %p", __func__, wait);
 
 	thr_ret = _wapi_handle_unlock_handle (wait);
 	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
 
-	if (apc_pending) {
-		_wapi_thread_dispatch_apc_queue (current_thread);
-		ret = WAIT_IO_COMPLETION;
-	}
-	
 	return(ret);
 }
 
-struct handle_cleanup_data
-{
-	guint32 numobjects;
-	gpointer *handles;
-};
-
-static void handle_cleanup (void *data)
-{
-	struct handle_cleanup_data *handles = (struct handle_cleanup_data *)data;
-
-	_wapi_handle_unlock_handles (handles->numobjects, handles->handles);
-}
-
 static gboolean test_and_own (guint32 numobjects, gpointer *handles,
 			      gboolean waitall, guint32 *count,
 			      guint32 *lowest)
 {
-	struct handle_cleanup_data cleanup_data;
 	gboolean done;
 	int i;
 	
-#ifdef DEBUG
-	g_message ("%s: locking handles", __func__);
-#endif
-	cleanup_data.numobjects = numobjects;
-	cleanup_data.handles = handles;
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: locking handles", __func__);
 	
-	pthread_cleanup_push (handle_cleanup, (void *)&cleanup_data);
 	done = _wapi_handle_count_signalled_handles (numobjects, handles,
 						     waitall, count, lowest);
 	if (done == TRUE) {
@@ -507,18 +423,13 @@ static gboolean test_and_own (guint32 numobjects, gpointer *handles,
 		}
 	}
 	
-#ifdef DEBUG
-	g_message ("%s: unlocking handles", __func__);
-#endif
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: unlocking handles", __func__);
 
-	/* calls the unlock function */
-	pthread_cleanup_pop (1);
+	_wapi_handle_unlock_handles (numobjects, handles);
 
 	return(done);
 }
 
-
-
 /**
  * WaitForMultipleObjectsEx:
  * @numobjects: The number of objects in @handles. The maximum allowed
@@ -553,16 +464,17 @@ guint32 WaitForMultipleObjectsEx(guint32 numobjects, gpointer *handles,
 				 gboolean waitall, guint32 timeout,
 				 gboolean alertable)
 {
-	GHashTable *dups;
 	gboolean duplicate = FALSE, bogustype = FALSE, done;
 	guint32 count, lowest;
-	struct timespec abstime;
 	guint i;
 	guint32 ret;
 	int thr_ret;
-	gpointer current_thread = _wapi_thread_handle_from_id (pthread_self ());
+	gpointer current_thread = wapi_get_current_thread_handle ();
 	guint32 retval;
 	gboolean poll;
+	gpointer sorted_handles [MAXIMUM_WAIT_OBJECTS];
+	gboolean apc_pending = FALSE;
+	gint64 now, end;
 	
 	if (current_thread == NULL) {
 		SetLastError (ERROR_INVALID_HANDLE);
@@ -570,9 +482,7 @@ guint32 WaitForMultipleObjectsEx(guint32 numobjects, gpointer *handles,
 	}
 	
 	if (numobjects > MAXIMUM_WAIT_OBJECTS) {
-#ifdef DEBUG
-		g_message ("%s: Too many handles: %d", __func__, numobjects);
-#endif
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Too many handles: %d", __func__, numobjects);
 
 		return(WAIT_FAILED);
 	}
@@ -582,17 +492,12 @@ guint32 WaitForMultipleObjectsEx(guint32 numobjects, gpointer *handles,
 	}
 
 	/* Check for duplicates */
-	dups = g_hash_table_new (g_direct_hash, g_direct_equal);
 	for (i = 0; i < numobjects; i++) {
-		gpointer exists;
-
 		if (handles[i] == _WAPI_THREAD_CURRENT) {
-			handles[i] = _wapi_thread_handle_from_id (pthread_self ());
+			handles[i] = wapi_get_current_thread_handle ();
 			
 			if (handles[i] == NULL) {
-#ifdef DEBUG
-				g_message ("%s: Handle %d bogus", __func__, i);
-#endif
+				MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Handle %d bogus", __func__, i);
 
 				bogustype = TRUE;
 				break;
@@ -600,59 +505,48 @@ guint32 WaitForMultipleObjectsEx(guint32 numobjects, gpointer *handles,
 		}
 
 		if ((GPOINTER_TO_UINT (handles[i]) & _WAPI_PROCESS_UNHANDLED) == _WAPI_PROCESS_UNHANDLED) {
-#ifdef DEBUG
-			g_message ("%s: Handle %d pseudo process", __func__,
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Handle %d pseudo process", __func__,
 				   i);
-#endif
 
 			bogustype = TRUE;
 			break;
 		}
-		
-		exists = g_hash_table_lookup (dups, handles[i]);
-		if (exists != NULL) {
-#ifdef DEBUG
-			g_message ("%s: Handle %p duplicated", __func__,
-				   handles[i]);
-#endif
-
-			duplicate = TRUE;
-			break;
-		}
 
 		if (_wapi_handle_test_capabilities (handles[i], WAPI_HANDLE_CAP_WAIT) == FALSE) {
-#ifdef DEBUG
-			g_message ("%s: Handle %p can't be waited for",
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Handle %p can't be waited for",
 				   __func__, handles[i]);
-#endif
 
 			bogustype = TRUE;
+			break;
 		}
 
-		g_hash_table_insert (dups, handles[i], handles[i]);
+		sorted_handles [i] = handles [i];
 		_wapi_handle_ops_prewait (handles[i]);
 	}
-	g_hash_table_destroy (dups);
+
+	qsort (sorted_handles, numobjects, sizeof (gpointer), g_direct_equal);
+	for (i = 1; i < numobjects; i++) {
+		if (sorted_handles [i - 1] == sorted_handles [i]) {
+			duplicate = TRUE;
+			break;
+		}
+	}
 
 	if (duplicate == TRUE) {
-#ifdef DEBUG
-		g_message ("%s: Returning due to duplicates", __func__);
-#endif
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Returning due to duplicates", __func__);
 
 		return(WAIT_FAILED);
 	}
 
 	if (bogustype == TRUE) {
-#ifdef DEBUG
-		g_message ("%s: Returning due to bogus type", __func__);
-#endif
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Returning due to bogus type", __func__);
 
 		return(WAIT_FAILED);
 	}
 
 	poll = FALSE;
 	for (i = 0; i < numobjects; ++i)
-		if (_wapi_handle_type (handles [i]) == WAPI_HANDLE_PROCESS)
+		if (_wapi_handle_type (handles [i]) == WAPI_HANDLE_PROCESS || _WAPI_SHARED_HANDLE (_wapi_handle_type (handles[i]))) 
 			/* Can't wait for a process handle + another handle without polling */
 			poll = TRUE;
 
@@ -664,18 +558,13 @@ guint32 WaitForMultipleObjectsEx(guint32 numobjects, gpointer *handles,
 	if (timeout == 0) {
 		return WAIT_TIMEOUT;
 	}
+
+	if (timeout != INFINITE)
+		end = mono_100ns_ticks () + timeout * 1000 * 10;
+
 	/* Have to wait for some or all handles to become signalled
 	 */
 
-	if(timeout!=INFINITE) {
-		_wapi_calc_timeout (&abstime, timeout);
-	}
-
-	if (alertable && _wapi_thread_apc_pending (current_thread)) {
-		_wapi_thread_dispatch_apc_queue (current_thread);
-		return WAIT_IO_COMPLETION;
-	}
-	
 	for (i = 0; i < numobjects; i++) {
 		/* Add a reference, as we need to ensure the handle wont
 		 * disappear from under us while we're waiting in the loop
@@ -692,15 +581,12 @@ guint32 WaitForMultipleObjectsEx(guint32 numobjects, gpointer *handles,
 			_wapi_handle_ops_prewait (handles[i]);
 		
 			if (_wapi_handle_test_capabilities (handles[i], WAPI_HANDLE_CAP_SPECIAL_WAIT) == TRUE && _wapi_handle_issignalled (handles[i]) == FALSE) {
-				_wapi_handle_ops_special_wait (handles[i], 0);
+				_wapi_handle_ops_special_wait (handles[i], 0, alertable);
 			}
 		}
 		
-#ifdef DEBUG
-		g_message ("%s: locking signal mutex", __func__);
-#endif
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: locking signal mutex", __func__);
 
-		pthread_cleanup_push ((void(*)(void *))_wapi_handle_unlock_signal_mutex, NULL);
 		thr_ret = _wapi_handle_lock_signal_mutex ();
 		g_assert (thr_ret == 0);
 
@@ -720,25 +606,26 @@ guint32 WaitForMultipleObjectsEx(guint32 numobjects, gpointer *handles,
 		if (!done) {
 			/* Enter the wait */
 			if (timeout == INFINITE) {
-				ret = _wapi_handle_wait_signal (poll);
+				ret = _wapi_handle_timedwait_signal (INFINITE, poll, &apc_pending);
 			} else {
-				ret = _wapi_handle_timedwait_signal (&abstime, poll);
+				now = mono_100ns_ticks ();
+				if (end < now) {
+					ret = WAIT_TIMEOUT;
+				} else {
+					ret = _wapi_handle_timedwait_signal ((end - now) / 10 / 1000, poll, &apc_pending);
+				}
 			}
 		} else {
 			/* No need to wait */
 			ret = 0;
 		}
 
-#ifdef DEBUG
-		g_message ("%s: unlocking signal mutex", __func__);
-#endif
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: unlocking signal mutex", __func__);
 
 		thr_ret = _wapi_handle_unlock_signal_mutex (NULL);
 		g_assert (thr_ret == 0);
-		pthread_cleanup_pop (0);
 		
-		if (alertable && _wapi_thread_apc_pending (current_thread)) {
-			_wapi_thread_dispatch_apc_queue (current_thread);
+		if (alertable && apc_pending) {
 			retval = WAIT_IO_COMPLETION;
 			break;
 		}
@@ -756,10 +643,8 @@ guint32 WaitForMultipleObjectsEx(guint32 numobjects, gpointer *handles,
 			/* Didn't get all handles, and there was a
 			 * timeout or other error
 			 */
-#ifdef DEBUG
-			g_message ("%s: wait returned error: %s", __func__,
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: wait returned error: %s", __func__,
 				   strerror (ret));
-#endif
 
 			if(ret==ETIMEDOUT) {
 				retval = WAIT_TIMEOUT;
